import React, { Fragment, useCallback, useEffect, useReducer, useState } from 'react';
import { useModal } from '@faceless-ui/modal';
import { useTranslation } from 'react-i18next';
import { getTranslation } from '../../../../utilities/getTranslation';
import { useConfig } from '../../utilities/Config';
import { useAuth } from '../../utilities/Auth';
import { DocumentInfoProvider } from '../../utilities/DocumentInfo';
import RenderCustomComponent from '../../utilities/RenderCustomComponent';
import usePayloadAPI from '../../../hooks/usePayloadAPI';
import DefaultList from '../../views/collections/List/Default';
import Label from '../../forms/Label';
import ReactSelect from '../ReactSelect';
import { useDocumentDrawer } from '../DocumentDrawer';
import Pill from '../Pill';
import X from '../../icons/X';
import ViewDescription from '../ViewDescription';
import getInitialColumnState from '../../views/collections/List/getInitialColumns';
import buildListColumns from '../../views/collections/List/buildColumns';
import formatFields from '../../views/collections/List/formatFields';
import { usePreferences } from '../../utilities/Preferences';
import { baseClass } from '.';
const buildColumns = ({ collectionConfig, columns, onSelect, t, }) => buildListColumns({
    collection: collectionConfig,
    columns,
    t,
    cellProps: [{
            link: false,
            onClick: ({ collection, rowData }) => {
                if (typeof onSelect === 'function') {
                    onSelect({
                        docID: rowData.id,
                        collectionConfig: collection,
                    });
                }
            },
            className: `${baseClass}__first-cell`,
        }],
});
const shouldIncludeCollection = ({ coll: { admin: { enableRichTextRelationship }, upload, slug, }, uploads, collectionSlugs, }) => (enableRichTextRelationship && ((uploads && Boolean(upload)) || (collectionSlugs === null || collectionSlugs === void 0 ? void 0 : collectionSlugs.includes(slug))));
export const ListDrawerContent = ({ drawerSlug, onSelect, customHeader, collectionSlugs, uploads, selectedCollection, }) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j;
    const { t, i18n } = useTranslation(['upload', 'general']);
    const { permissions } = useAuth();
    const { getPreference, setPreference } = usePreferences();
    const { isModalOpen, closeModal } = useModal();
    const [limit, setLimit] = useState();
    const [sort, setSort] = useState(null);
    const [page, setPage] = useState(1);
    const [where, setWhere] = useState(null);
    const { serverURL, routes: { api }, collections } = useConfig();
    const [enabledCollectionConfigs] = useState(() => collections.filter((coll) => shouldIncludeCollection({ coll, uploads, collectionSlugs })));
    const [selectedCollectionConfig, setSelectedCollectionConfig] = useState(() => {
        let initialSelection;
        if (selectedCollection) {
            // if passed a selection, find it and check if it's enabled
            const foundSelection = collections.find(({ slug }) => slug === selectedCollection);
            if (foundSelection && shouldIncludeCollection({ coll: foundSelection, uploads, collectionSlugs })) {
                initialSelection = foundSelection;
            }
        }
        else {
            // return the first one that is enabled
            initialSelection = collections.find((coll) => shouldIncludeCollection({ coll, uploads, collectionSlugs }));
        }
        return initialSelection;
    });
    const [selectedOption, setSelectedOption] = useState(() => (selectedCollectionConfig ? { label: getTranslation(selectedCollectionConfig.labels.singular, i18n), value: selectedCollectionConfig.slug } : undefined));
    const [fields, setFields] = useState(() => formatFields(selectedCollectionConfig, t));
    const [tableColumns, setTableColumns] = useState(() => {
        const initialColumns = getInitialColumnState(fields, selectedCollectionConfig.admin.useAsTitle, selectedCollectionConfig.admin.defaultColumns);
        return buildColumns({
            collectionConfig: selectedCollectionConfig,
            columns: initialColumns,
            t,
            onSelect,
        });
    });
    // allow external control of selected collection, same as the initial state logic above
    useEffect(() => {
        let newSelection;
        if (selectedCollection) {
            // if passed a selection, find it and check if it's enabled
            const foundSelection = collections.find(({ slug }) => slug === selectedCollection);
            if (foundSelection && shouldIncludeCollection({ coll: foundSelection, uploads, collectionSlugs })) {
                newSelection = foundSelection;
            }
        }
        else {
            // return the first one that is enabled
            newSelection = collections.find((coll) => shouldIncludeCollection({ coll, uploads, collectionSlugs }));
        }
        setSelectedCollectionConfig(newSelection);
    }, [selectedCollection, collectionSlugs, uploads, collections, onSelect, t]);
    const activeColumnNames = tableColumns.map(({ accessor }) => accessor);
    const stringifiedActiveColumns = JSON.stringify(activeColumnNames);
    const preferenceKey = `${selectedCollectionConfig.slug}-list`;
    // this is the 'create new' drawer
    const [DocumentDrawer, DocumentDrawerToggler, { drawerSlug: documentDrawerSlug, },] = useDocumentDrawer({
        collectionSlug: selectedCollectionConfig.slug,
    });
    useEffect(() => {
        if (selectedOption) {
            setSelectedCollectionConfig(collections.find(({ slug }) => selectedOption.value === slug));
        }
    }, [selectedOption, collections]);
    const collectionPermissions = (_a = permissions === null || permissions === void 0 ? void 0 : permissions.collections) === null || _a === void 0 ? void 0 : _a[selectedCollectionConfig === null || selectedCollectionConfig === void 0 ? void 0 : selectedCollectionConfig.slug];
    const hasCreatePermission = (_b = collectionPermissions === null || collectionPermissions === void 0 ? void 0 : collectionPermissions.create) === null || _b === void 0 ? void 0 : _b.permission;
    // If modal is open, get active page of upload gallery
    const isOpen = isModalOpen(drawerSlug);
    const apiURL = isOpen ? `${serverURL}${api}/${selectedCollectionConfig.slug}` : null;
    const [cacheBust, dispatchCacheBust] = useReducer((state) => state + 1, 0); // used to force a re-fetch even when apiURL is unchanged
    const [{ data, isError }, { setParams }] = usePayloadAPI(apiURL, {});
    const moreThanOneAvailableCollection = enabledCollectionConfigs.length > 1;
    useEffect(() => {
        const params = {};
        if (page)
            params.page = page;
        if (where)
            params.where = where;
        if (sort)
            params.sort = sort;
        if (limit)
            params.limit = limit;
        if (cacheBust)
            params.cacheBust = cacheBust;
        setParams(params);
    }, [setParams, page, sort, where, limit, cacheBust]);
    useEffect(() => {
        const syncColumnsFromPrefs = async () => {
            const currentPreferences = await getPreference(preferenceKey);
            const newFields = formatFields(selectedCollectionConfig, t);
            setFields(newFields);
            const initialColumns = getInitialColumnState(newFields, selectedCollectionConfig.admin.useAsTitle, selectedCollectionConfig.admin.defaultColumns);
            setTableColumns(buildColumns({
                collectionConfig: selectedCollectionConfig,
                columns: (currentPreferences === null || currentPreferences === void 0 ? void 0 : currentPreferences.columns) || initialColumns,
                t,
                onSelect,
            }));
        };
        syncColumnsFromPrefs();
    }, [t, getPreference, preferenceKey, onSelect, selectedCollectionConfig]);
    useEffect(() => {
        const newPreferences = {
            limit,
            sort,
            columns: JSON.parse(stringifiedActiveColumns),
        };
        setPreference(preferenceKey, newPreferences);
    }, [sort, limit, stringifiedActiveColumns, setPreference, preferenceKey]);
    const setActiveColumns = useCallback((columns) => {
        setTableColumns(buildColumns({
            collectionConfig: selectedCollectionConfig,
            columns,
            t,
            onSelect,
        }));
    }, [selectedCollectionConfig, t, onSelect]);
    const onCreateNew = useCallback(({ doc }) => {
        if (typeof onSelect === 'function') {
            onSelect({
                docID: doc.id,
                collectionConfig: selectedCollectionConfig,
            });
        }
        dispatchCacheBust();
        closeModal(documentDrawerSlug);
        closeModal(drawerSlug);
    }, [closeModal, documentDrawerSlug, drawerSlug, onSelect, selectedCollectionConfig]);
    if (!selectedCollectionConfig || isError) {
        return null;
    }
    return (React.createElement(Fragment, null,
        React.createElement(DocumentInfoProvider, { collection: selectedCollectionConfig },
            React.createElement(RenderCustomComponent, { DefaultComponent: DefaultList, CustomComponent: (_e = (_d = (_c = selectedCollectionConfig === null || selectedCollectionConfig === void 0 ? void 0 : selectedCollectionConfig.admin) === null || _c === void 0 ? void 0 : _c.components) === null || _d === void 0 ? void 0 : _d.views) === null || _e === void 0 ? void 0 : _e.List, componentProps: {
                    collection: {
                        ...selectedCollectionConfig,
                        fields,
                    },
                    customHeader: (React.createElement("header", { className: `${baseClass}__header` },
                        React.createElement("div", { className: `${baseClass}__header-wrap` },
                            React.createElement("div", { className: `${baseClass}__header-content` },
                                React.createElement("h2", { className: `${baseClass}__header-text` }, !customHeader ? getTranslation((_f = selectedCollectionConfig === null || selectedCollectionConfig === void 0 ? void 0 : selectedCollectionConfig.labels) === null || _f === void 0 ? void 0 : _f.plural, i18n) : customHeader),
                                hasCreatePermission && (React.createElement(DocumentDrawerToggler, { className: `${baseClass}__create-new-button` },
                                    React.createElement(Pill, null, t('general:createNew'))))),
                            React.createElement("button", { type: "button", onClick: () => {
                                    closeModal(drawerSlug);
                                }, className: `${baseClass}__header-close` },
                                React.createElement(X, null))),
                        ((_g = selectedCollectionConfig === null || selectedCollectionConfig === void 0 ? void 0 : selectedCollectionConfig.admin) === null || _g === void 0 ? void 0 : _g.description) && (React.createElement("div", { className: `${baseClass}__sub-header` },
                            React.createElement(ViewDescription, { description: selectedCollectionConfig.admin.description }))),
                        moreThanOneAvailableCollection && (React.createElement("div", { className: `${baseClass}__select-collection-wrap` },
                            React.createElement(Label, { label: t('selectCollectionToBrowse') }),
                            React.createElement(ReactSelect, { className: `${baseClass}__select-collection`, value: selectedOption, onChange: setSelectedOption, options: enabledCollectionConfigs.map((coll) => ({ label: getTranslation(coll.labels.singular, i18n), value: coll.slug })) }))))),
                    data,
                    limit: limit || ((_j = (_h = selectedCollectionConfig === null || selectedCollectionConfig === void 0 ? void 0 : selectedCollectionConfig.admin) === null || _h === void 0 ? void 0 : _h.pagination) === null || _j === void 0 ? void 0 : _j.defaultLimit),
                    setLimit,
                    tableColumns,
                    setColumns: setActiveColumns,
                    setSort,
                    newDocumentURL: null,
                    hasCreatePermission,
                    columnNames: activeColumnNames,
                    disableEyebrow: true,
                    modifySearchParams: false,
                    onCardClick: (doc) => {
                        if (typeof onSelect === 'function') {
                            onSelect({
                                docID: doc.id,
                                collectionConfig: selectedCollectionConfig,
                            });
                        }
                        closeModal(drawerSlug);
                    },
                    disableCardLink: true,
                    handleSortChange: setSort,
                    handleWhereChange: setWhere,
                    handlePageChange: setPage,
                    handlePerPageChange: setLimit,
                } })),
        React.createElement(DocumentDrawer, { onSave: onCreateNew })));
};
