import React, { useCallback, useEffect, useId, useMemo, useState } from 'react';
import { useModal } from '@faceless-ui/modal';
import { Drawer, DrawerToggler } from '../Drawer';
import { useEditDepth } from '../../utilities/EditDepth';
import { ListDrawerContent } from './DrawerContent';
import './index.scss';
export const baseClass = 'list-drawer';
const formatListDrawerSlug = ({ depth, uuid, }) => `list-drawer_${depth}_${uuid}`;
export const ListDrawerToggler = ({ children, className, drawerSlug, disabled, ...rest }) => {
    return (React.createElement(DrawerToggler, { slug: drawerSlug, formatSlug: false, className: [
            className,
            `${baseClass}__toggler`,
        ].filter(Boolean).join(' '), disabled: disabled, ...rest }, children));
};
export const ListDrawer = (props) => {
    const { drawerSlug } = props;
    return (React.createElement(Drawer, { slug: drawerSlug, formatSlug: false, className: baseClass },
        React.createElement(ListDrawerContent, { ...props })));
};
export const useListDrawer = ({ collectionSlugs, uploads, selectedCollection }) => {
    const drawerDepth = useEditDepth();
    const uuid = useId();
    const { modalState, toggleModal, closeModal, openModal } = useModal();
    const [isOpen, setIsOpen] = useState(false);
    const drawerSlug = formatListDrawerSlug({
        depth: drawerDepth,
        uuid,
    });
    useEffect(() => {
        var _a;
        setIsOpen(Boolean((_a = modalState[drawerSlug]) === null || _a === void 0 ? void 0 : _a.isOpen));
    }, [modalState, drawerSlug]);
    const toggleDrawer = useCallback(() => {
        toggleModal(drawerSlug);
    }, [toggleModal, drawerSlug]);
    const closeDrawer = useCallback(() => {
        closeModal(drawerSlug);
    }, [drawerSlug, closeModal]);
    const openDrawer = useCallback(() => {
        openModal(drawerSlug);
    }, [drawerSlug, openModal]);
    const MemoizedDrawer = useMemo(() => {
        return ((props) => (React.createElement(ListDrawer, { ...props, drawerSlug: drawerSlug, collectionSlugs: collectionSlugs, uploads: uploads, closeDrawer: closeDrawer, key: drawerSlug, selectedCollection: selectedCollection })));
    }, [drawerSlug, collectionSlugs, uploads, closeDrawer, selectedCollection]);
    const MemoizedDrawerToggler = useMemo(() => {
        return ((props) => (React.createElement(ListDrawerToggler, { ...props, drawerSlug: drawerSlug })));
    }, [drawerSlug]);
    const MemoizedDrawerState = useMemo(() => ({
        drawerSlug,
        drawerDepth,
        isDrawerOpen: isOpen,
        toggleDrawer,
        closeDrawer,
        openDrawer,
    }), [drawerDepth, drawerSlug, isOpen, toggleDrawer, closeDrawer, openDrawer]);
    return [
        MemoizedDrawer,
        MemoizedDrawerToggler,
        MemoizedDrawerState,
    ];
};
